/**
 * (C) 2010 jolira (http://www.jolira.com). Licensed under the GNU General Public License, Version 3.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 * http://www.gnu.org/licenses/gpl-3.0-standalone.html Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language governing permissions and limitations
 * under the License.
 */
package com.google.code.joliratools.wizard;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.wicket.extensions.wizard.WizardStep;
import org.apache.wicket.markup.html.WebPage;
import org.apache.wicket.request.target.coding.IRequestTargetUrlCodingStrategy;
import org.apache.wicket.request.target.coding.QueryStringUrlCodingStrategy;

import com.google.inject.Binder;
import com.google.inject.Module;
import com.google.inject.binder.LinkedBindingBuilder;
import com.google.inject.multibindings.Multibinder;

/**
 * @author "Gabe Hopper"
 * 
 */
public abstract class GuicyWizardModule implements Module {
    protected static final <T extends WebPage> void mount(
            final Multibinder<IRequestTargetUrlCodingStrategy> strategiesBinder, final String path,
            final Class<T> pageCls) {
        final IRequestTargetUrlCodingStrategy strategy = new QueryStringUrlCodingStrategy(path, pageCls);
        final LinkedBindingBuilder<IRequestTargetUrlCodingStrategy> builder = strategiesBinder.addBinding();

        builder.toInstance(strategy);
    }

    private final List<Class<? extends WizardStep>> steps = new ArrayList<Class<? extends WizardStep>>();

    public void configure(final Binder binder) {

        confiureMount(configureStrategiesBinder(binder));
        configureSteps(binder);
        configureServices(binder);
        configureWizard(binder);
        configureModel(binder);
        configureState(binder);
    }

    protected void configureModel(final Binder binder) {
        binder.bind(IGuicyWizardModel.class).to(getModelClass());
    }

    protected abstract void configureServices(final Binder binder);

    protected void configureState(final Binder binder) {
        binder.bind(IGuicyWizardState.class).to(getStateClass());
    }

    protected void configureSteps(final Binder binder) {
        final Multibinder<WizardStep> wizardBinder = Multibinder.newSetBinder(binder, WizardStep.class);
        this.initStepClasses();
        for (Class<? extends WizardStep> step : getSteps()) {
            wizardBinder.addBinding().to(step);
        }

    }

    protected Multibinder<IRequestTargetUrlCodingStrategy> configureStrategiesBinder(final Binder binder) {
        final Multibinder<IRequestTargetUrlCodingStrategy> strategiesBinder = Multibinder.newSetBinder(binder,
                IRequestTargetUrlCodingStrategy.class);
        return strategiesBinder;
    }

    protected void configureWizard(final Binder binder) {
        binder.bind(IGuicyWizard.class).to(getWizardClass());
        binder.bind(IGuicyWizardPage.class).to(getWizardPage());
    }

    protected void confiureMount(final Multibinder<IRequestTargetUrlCodingStrategy> conficStrategiesBinder) {
        final Map<String, Class<? extends WebPage>> mounts = getMounts();
        if (mounts != null) {
            Set<String> keys = mounts.keySet();
            for (String key : keys) {
                mount(conficStrategiesBinder, key, mounts.get(key));
            }
        }
    }

    protected abstract Class<? extends IGuicyWizardModel> getModelClass();

    protected abstract Map<String, Class<? extends WebPage>> getMounts();

    protected abstract Class<? extends IGuicyWizardState> getStateClass();

    protected List<Class<? extends WizardStep>> getSteps() {
        return steps;
    }

    protected abstract Class<? extends IGuicyWizard> getWizardClass();

    protected abstract Class<? extends IGuicyWizardPage> getWizardPage();

    protected abstract void initStepClasses();

}
